package plugins.ArtistAliasCompletion;

import info.clearthought.layout.TableLayout;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Locale;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.text.View;

import allrounder.DbHandler;
import allrounder.DirHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;
import allrounder.logger.Logwriter;
import allrounder.plugin.YiffIndexPlugin;

public class ArtistAliasCompletion implements YiffIndexPlugin {
	TreeMap<String, String> mLanguageMap = null;

	public ArtistAliasCompletion() {
		mLanguageMap = StringLib.getLanguageMapBySystemLocale(DirHandler.getDirOfClass(getClass()), Locale.ENGLISH);
	}

	public String getName() {
		return mLanguageMap.get("title");
	}

	public String getDescription() {
		return mLanguageMap.get("description");
	}

	public void pluginCalled(JFrame mainFrame) {
		new ArtistAliasCompletionDialog(mLanguageMap, mainFrame);
	}
}

class ArtistAliasCompletionDialog extends JDialog {
	TreeMap<String, String> mLanguageMap;
	ArrayList<ArrayList<String> > mArtistAliases;
	
	JLabel lblDescr = new JLabel();
	JLabel lblBrackExpl1 = new JLabel();
	JLabel lblBrackExpl2 = new JLabel();
	JTable tblAliases;
	JButton btnComplete = new JButton();
	JButton btnClose = new JButton();
	
	public ArtistAliasCompletionDialog(TreeMap<String, String> languageMap, JFrame mainFrame) {
		if ((mLanguageMap = languageMap) == null) return;
		mLanguageMap = languageMap;
		
		if (!readArtistAliases()) return;
		
		this.setTitle(mLanguageMap.get("title"));
		lblDescr.setText("<html><body>" + mLanguageMap.get("description") + "</body></html>");
		lblBrackExpl1.setText("<html><body>" + mLanguageMap.get("bracketexpl1") + "</body></html>");
		lblBrackExpl2.setText("<html><body>" + mLanguageMap.get("bracketexpl2") + "</body></html>");
		btnComplete.setText(mLanguageMap.get("complete"));
		btnComplete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ArtistAliasCompletionDialog.this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				int i = addAliases();
				ArtistAliasCompletionDialog.this.setCursor(Cursor.getDefaultCursor());
				JOptionPane.showMessageDialog(ArtistAliasCompletionDialog.this, mLanguageMap.get("resulttext") + " " + i, mLanguageMap.get("resulttitle"), JOptionPane.INFORMATION_MESSAGE);
			}
		});
		btnClose.setText(mLanguageMap.get("close"));
		btnClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				setVisible(false);
				dispose();
			}
		});
		
		// build map data
		String[] columnNames = {mLanguageMap.get("tnick"), mLanguageMap.get("talias")};
		// converting the ArrayList<ArrayList<String> > to string[][]
		TreeMap<AACTableString, String> tmpmap = new TreeMap<AACTableString, String>();
		for (ArrayList<String> aliases : mArtistAliases) {
			for (String sNick : aliases) {
				StringBuilder sAliasList = null;
				for (String sAlias : aliases) {
					if (sAlias.equals(sNick)) continue;
					if (sAliasList== null) {
						sAliasList = new StringBuilder(sAlias);
					} else {
						sAliasList.append(", " + sAlias);
					}
				}
				tmpmap.put(new AACTableString(sNick), sAliasList.toString());
			}
		}
		Object[][] data = new Object[tmpmap.size()][2];
		Iterator<AACTableString> iter = tmpmap.keySet().iterator();
		for (int i = 0; i < tmpmap.size(); ++i) {
			//AACTableString s = iter.next();
			data[i][0] = iter.next();
			data[i][1] = tmpmap.get(data[i][0]);
		}
		tmpmap = null;
		
		tblAliases = new JTable(data, columnNames) {
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		tblAliases.setAutoCreateRowSorter(true);
		tblAliases.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		tblAliases.getColumnModel().getColumn(0).setPreferredWidth(120);
		tblAliases.getColumnModel().getColumn(1).setPreferredWidth(330);
		
		((JPanel)this.getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		setSize(500, 300);
		setLocationRelativeTo(mainFrame);
		setResizable(false);
		
		//column / row
		double[][] layout = {{btnComplete.getPreferredSize().width, TableLayout.FILL, btnClose.getPreferredSize().width},
							{getPreferredHeight(lblDescr, 450), 10, getPreferredHeight(lblBrackExpl1, 450), 10, 
							getPreferredHeight(lblBrackExpl2, 450), 10, TableLayout.FILL, 10, btnComplete.getPreferredSize().height}};
		this.setLayout(new TableLayout(layout));

		this.add(lblDescr, "0, 0, 2, 0");
		this.add(lblBrackExpl1, "0, 2, 2, 2");
		this.add(lblBrackExpl2, "0, 4, 2, 2");
		this.add(new JScrollPane(tblAliases), "0, 6, 2, 4");
		this.add(btnComplete, "0, 8");
		this.add(btnClose, "2, 8");
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setVisible(true);
	}
	
	private static double getPreferredHeight(JLabel html, int prefSize) {
		View view = (View) html.getClientProperty(javax.swing.plaf.basic.BasicHTML.propertyKey);

		view.setSize(prefSize, 0);
		float h = view.getPreferredSpan(View.Y_AXIS);

		return Math.ceil(h);
	}
	
	private boolean readArtistAliases() {
		File fAAFile = new File(DirHandler.getDirOfClass(getClass()), "ArtistAliases.txt");
		
		mArtistAliases = new ArrayList<ArrayList<String> >();
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fAAFile), "UTF8"));
	        String str;
	        while ((str = in.readLine()) != null) {
	        	str = str.trim();
	        	if (str.startsWith("#")) continue;
	        	if (str.indexOf(";") == -1) continue;
	        	ArrayList<String> tmp = new ArrayList<String>(Arrays.asList(str.split(";")));
	        	mArtistAliases.add(tmp);
	        }
	        in.close();
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	e.printStackTrace(Logwriter.getInstance().getPrintStream());
	    	return false;
	    }
	    return true;
	}
	
	private int addAliases() {
		DbHandler DB = DbHandler.getInstance();
		ArrayList<String> alAllArtistsInDb = DB.getArtists();
		int iReturn = 0;
		for (ArrayList<String> alAliases : mArtistAliases) {
			//check if any of the aliases for this artist is even in the db
			// if yes, add all imagedata-ids of that alias
			TreeSet<Integer> imageDataIds = new TreeSet<Integer>();
			for (String alias : alAliases) {
				if (alias.charAt(0) == '{' && alias.charAt(alias.length() -1) == '}') continue;
				if (alias.charAt(0) == '[' && alias.charAt(alias.length() -1) == ']') {
					alias = alias.substring(1, alias.length() - 1);
				}
				boolean bInDb = false;
				for (String aliasInDB : alAllArtistsInDb) {
					if (aliasInDB.equalsIgnoreCase(alias)) {
						bInDb = true;
						break;
					}
				}
				if (!bInDb) continue;
				imageDataIds.addAll(DB.getImagedataIdsByArtist(alias));
			}
			if (imageDataIds.size() == 0) continue;
			
			// remove the aliases off the images and add the new ones
			for (int iImagedataId : imageDataIds) {
				TreeSet<String> artistsOfImage = new TreeSet<String>(String.CASE_INSENSITIVE_ORDER);
				TaggedImage img = DB.getIncompleteImageFromImageDataId(iImagedataId);
				TreeSet<String> artistsOfImageTmp = img.artists;
				artistsOfImage.addAll(artistsOfImageTmp);
				
				// check if img has to be changed
				boolean bChangeIt = false;
				for (String alias : alAliases) {
					if (alias.charAt(0) == '{' && alias.charAt(alias.length() -1) == '}') continue;
					if (alias.charAt(0) == '[' && alias.charAt(alias.length() -1) == ']') {
						bChangeIt = artistsOfImage.contains(alias.substring(1, alias.length() - 1));
					} else {
						bChangeIt = !artistsOfImageTmp.contains(alias);
					}
					if (bChangeIt) break;
				}
				if (!bChangeIt) continue;
				artistsOfImageTmp = null;
				
				++iReturn;
				//remove
				for (String alias : alAliases) {
					if ((alias.charAt(0) == '[' && alias.charAt(alias.length() -1) == ']')
							|| (alias.charAt(0) == '{' && alias.charAt(alias.length() -1) == '}')) {
						alias = alias.substring(1, alias.length() - 1);
					}
					artistsOfImage.remove(alias);
				}
				DB.removeAllArtistsFromImagedata(iImagedataId);
				
				// add
				for (String alias : alAliases) {
					if (alias.charAt(0) == '[' && alias.charAt(alias.length() -1) == ']') continue;
					if (alias.charAt(0) == '{' && alias.charAt(alias.length() -1) == '}') {
						artistsOfImage.add(alias.substring(1, alias.length() - 1));
					} else {
						artistsOfImage.add(alias);
					}
				}
				DB.addArtistsToImagedata(artistsOfImage, iImagedataId);
				
				DB.setImageChanged(img.hash, true);
			}
		}
		return iReturn;
	}
}


class AACTableString implements Comparable<AACTableString> {
	String sComparestring;
	String sDisplayString;

	public AACTableString(String s) {
		sComparestring = s;
		sDisplayString = s;
		if (sComparestring.charAt(0) == '[' && sComparestring.charAt(sComparestring.length() -1) == ']') {
			sComparestring = sComparestring.substring(1, sComparestring.length() - 1);
		}
	}
	
	public String toString() {
		return sDisplayString;
	}
	
	public int compareTo(AACTableString o) {
		return sComparestring.compareToIgnoreCase(o.sComparestring);
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof AACTableString)) return false;
		return (sComparestring.equalsIgnoreCase(((AACTableString)obj).sComparestring));
	}
}